@@ -30,6 +30,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -43,6 +44,10 @@
 #include "e-cal-backend-groupwise-utils.h"
 #include "e-gw-connection.h"
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 /* Private part of the CalBackendGroupwise structure */
 struct _ECalBackendGroupwisePrivate {
 	/* A mutex to control access to the private structure */
@@ -72,6 +77,8 @@ struct _ECalBackendGroupwisePrivate {
 static void e_cal_backend_groupwise_dispose (GObject *object);
 static void e_cal_backend_groupwise_finalize (GObject *object);
 static void sanitize_component (ECalBackendSync *backend, ECalComponent *comp, char *server_uid);
+static ECalBackendSyncStatus
+e_cal_backend_groupwise_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj);
 
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
 static ECalBackendClass *parent_class = NULL;
@@ -382,13 +389,11 @@ get_deltas (gpointer handle)
 		item_list = NULL;
 	}
 	
-	/* TODO currently the read cursors response does not give us the recurrencKey, uncomment
-	   this once the  response gives the recurrenceKey */
 	/* handle deleted items here by going over the entire cache and
 	 * checking for deleted items.*/
 	position = E_GW_CURSOR_POSITION_END;
 	cursor = 0;
-	status = e_gw_connection_create_cursor (cnc, cbgw->priv->container_id, "id iCalId recurrenceKey", NULL, &cursor);
+	status = e_gw_connection_create_cursor (cnc, cbgw->priv->container_id, "id iCalId recurrenceKey startDate", NULL, &cursor);
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) {
@@ -416,18 +421,6 @@ get_deltas (gpointer handle)
 			return TRUE;
 		}
 		
-		/* FIXME handle deleted items here by going over the entire cache and
-		 * checking for deleted items.*/
-#if 0
-		for (l1 = item_list; l1; l1 = g_list_next (l1)) {
-			char *icalid;
-			icalid = (char *)(l1->data);
-			cache_keys = g_slist_delete_link (cache_keys, 
-					g_slist_find_custom (cache_keys, icalid, (GCompareFunc) strcmp));
-			if (l1->data)
-				g_free (l1->data);
-		}	
-#endif
 		if (!item_list  || g_list_length (item_list) == 0)
 			done = TRUE;
 		else {
@@ -445,7 +438,41 @@ get_deltas (gpointer handle)
 	e_gw_connection_destroy_cursor (cnc, cbgw->priv->container_id, cursor);
 	e_file_cache_freeze_changes (E_FILE_CACHE (cache));
 
-#if 0
+	for (l = total_list; l != NULL; l = l->next) {
+		EGwItemCalId *calid = (EGwItemCalId *)	l->data;
+		GCompareFunc func = NULL;
+		GSList *remove = NULL;
+		char *real_key = NULL;
+
+		if (calid->recur_key && calid->ical_id) {
+			const char *rid = NULL;
+			icaltimetype tt = icaltime_from_string (calid->ical_id);
+			if (!tt.is_date) {
+				tt = icaltime_convert_to_zone (tt, priv->default_zone); 
+				icaltime_set_timezone (&tt, priv->default_zone);
+				rid = icaltime_as_ical_string (tt);
+			} else
+				rid = calid->ical_id;
+			real_key = g_strconcat (calid->recur_key, "@", rid, NULL);
+		}
+		
+		if (!calid->recur_key || real_key) 
+			func = (GCompareFunc) strcmp;
+		else
+			func = (GCompareFunc) compare_prefix;
+
+		if (!(remove = g_slist_find_custom (cache_keys, calid->recur_key ? real_key :
+						calid->ical_id,  func))) {
+			g_ptr_array_add (uid_array, g_strdup (calid->item_id));
+			needs_to_get = TRUE;
+		} else  {
+			cache_keys = g_slist_delete_link (cache_keys, remove);
+			g_message ("******** found \n");
+		}
+	
+		g_free (real_key);
+	}
+
 	for (l = cache_keys; l ; l = g_slist_next (l)) {
 		/* assumes rid is null - which works for now */
 		ECalComponent *comp = NULL;
@@ -460,36 +487,19 @@ get_deltas (gpointer handle)
 		vtype = e_cal_component_get_vtype (comp);
 		if ((vtype == E_CAL_COMPONENT_EVENT) ||
 				(vtype == E_CAL_COMPONENT_TODO)) {
+			char *comp_str = NULL;
+			ECalComponentId *id = e_cal_component_get_id (comp);
+			
 			comp_str = e_cal_component_get_as_string (comp);
 			e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), 
-					(char *) l->data, comp_str, NULL);
-			e_cal_backend_cache_remove_component (cache, (const char *) l->data, NULL);
+					id, comp_str, NULL);
+			e_cal_backend_cache_remove_component (cache, (const char *) id->uid, id->rid);
+
+			e_cal_component_free_id (id);
 			g_free (comp_str);
 		}
 		g_object_unref (comp);
 	}
-#endif
-
-	for (l = total_list; l != NULL; l = l->next) {
-		EGwItemCalId *calid = (EGwItemCalId *)	l->data;
-		GCompareFunc func = NULL;
-		GSList *remove = NULL;
-
-		if (calid->ical_id) 
-			func = (GCompareFunc) strcmp;
-		else
-			func = (GCompareFunc) compare_prefix;
-
-		if (!(remove = g_slist_find_custom (cache_keys, calid->ical_id ? calid->ical_id :
-						calid->recur_key,  func))) {
-			g_ptr_array_add (uid_array, g_strdup (calid->item_id));
-			needs_to_get = TRUE;
-		} else  {
-			cache_keys = g_slist_delete_link (cache_keys, remove);
-			continue;
-		}
-
-	}
 
 	if (needs_to_get) {
 		e_gw_connection_get_items_from_ids (priv->cnc,
@@ -520,7 +530,7 @@ get_deltas (gpointer handle)
 
 			g_object_unref (item);
 		}
-	}
+	} 
 
 	e_file_cache_thaw_changes (E_FILE_CACHE (cache));
 
@@ -841,6 +851,7 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 
 		if (E_IS_GW_CONNECTION (priv->cnc)) {
 			ECalBackendSyncStatus status;
+			
 			/* get the ID for the container */
 			if (priv->container_id)
 				g_free (priv->container_id);
@@ -856,6 +867,9 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 				return GNOME_Evolution_Calendar_OtherError;
 			}
 
+			e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
+			e_cal_backend_groupwise_add_timezone (E_CAL_BACKEND_SYNC (cbgw), NULL, (const char *) icaltimezone_get_tzid (priv->default_zone));
+
 			/* spawn a new thread for opening the calendar */
 			thread = g_thread_create ((GThreadFunc) cache_init, cbgw, FALSE, &error);
 			if (!thread) {
@@ -1068,6 +1082,7 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
 	ECalBackendSyncStatus status;
+	char *filename;
 	char *mangled_uri;
 	int i;
 	
@@ -1101,6 +1116,9 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 			}
 		}
 		
+		e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
+		e_cal_backend_groupwise_add_timezone (backend, cal, (const char *) icaltimezone_get_tzid (priv->default_zone));
+
 		g_mutex_unlock (priv->mutex);	
 		return GNOME_Evolution_Calendar_Success;
 	}
@@ -1119,10 +1137,14 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 		}
 	}
 
-	priv->local_attachments_store = 
-		g_strconcat ("file://", g_get_home_dir (), "/", ".evolution/cache/calendar",
-			     "/", mangled_uri, NULL);
+	filename = g_build_filename (g_get_home_dir (),
+				     ".evolution/cache/calendar",
+				     mangled_uri,
+				     NULL);
 	g_free (mangled_uri);
+	priv->local_attachments_store = 
+		g_filename_to_uri (filename, NULL, NULL);
+	g_free (filename);
 
 	/* FIXME: no need to set it online here when we implement the online/offline stuff correctly */
 	status = connect_to_server (cbgw);
@@ -1361,9 +1383,6 @@ e_cal_backend_groupwise_set_default_timezone (ECalBackendSync *backend, EDataCal
 	/* Set the default timezone to it. */
 	priv->default_zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
 
-	/* FIXME  write it into the cache*/
-	e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1942,21 +1961,19 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		ECalBackendSyncStatus status;
 		const char *id_to_remove = NULL;
 		icalcomponent *icalcomp;
+	
+		status = e_cal_backend_groupwise_get_object (backend, cal, uid, rid, &calobj);
+		if (status != GNOME_Evolution_Calendar_Success)
+			return status;
+		g_message ("object found \n");
+
+		icalcomp = icalparser_parse_string (calobj);
+		if (!icalcomp) {
+			g_free (calobj);
+			return GNOME_Evolution_Calendar_InvalidObject;
+		}
 
 		if (mod == CALOBJ_MOD_THIS) {
-
-			status = e_cal_backend_groupwise_get_object (backend, cal, uid, rid, &calobj);
-			if (status != GNOME_Evolution_Calendar_Success)
-				return status;
-
-			*old_object = strdup (calobj);
-
-			icalcomp = icalparser_parse_string (calobj);
-			if (!icalcomp) {
-				g_free (calobj);
-				return GNOME_Evolution_Calendar_InvalidObject;
-			}
-		
 			id_to_remove = get_gw_item_id (icalcomp); 
 			if (!id_to_remove) {
 				/* use the iCalId to remove the object */
@@ -1977,6 +1994,7 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 					return GNOME_Evolution_Calendar_ObjectNotFound;
 				}
 				*object = NULL;
+				*old_object = strdup (calobj);
 				g_free (calobj);
 				return GNOME_Evolution_Calendar_Success;
 			} else {
@@ -1996,6 +2014,7 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 					status = e_gw_connection_decline_request (priv->cnc, id_to_remove, NULL, uid);
 			} else {
 				GList *item_ids = NULL;	
+				
 				for (l = comp_list; l; l = l->next) {
 					ECalComponent *comp = E_CAL_COMPONENT (l->data);
 
@@ -2012,18 +2031,26 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 
 				for (l = comp_list; l; l = l->next) {
 					ECalComponent *comp = E_CAL_COMPONENT (l->data);
-					e_cal_backend_cache_remove_component (priv->cache, uid, 
-							e_cal_component_get_recurid_as_string (comp));
-					e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), 
-							     uid, e_cal_component_get_as_string (comp), NULL);
+					ECalComponentId *id = e_cal_component_get_id (comp);
+
+					e_cal_backend_cache_remove_component (priv->cache, id->uid, 
+							id->rid);
+					if (!g_str_equal (id->rid, rid))
+						e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), id, e_cal_component_get_as_string (comp), NULL);
+					e_cal_component_free_id (id);
+					
 					g_object_unref (comp);
 				
 				}
 				/* Setting NULL would trigger another signal.
 				 * We do not set the *object to NULL  */
 				g_slist_free (comp_list);
+				*old_object = strdup (calobj);
+				*object = NULL;
+				g_free (calobj);
 				return GNOME_Evolution_Calendar_Success;
 			} else {
+				g_free (calobj);
 				return GNOME_Evolution_Calendar_OtherError;
 			}
 		} else
@@ -2036,80 +2063,133 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	
 }
 
+/* This function is largely duplicated in
+ * ../file/e-cal-backend-file.c
+ */
 static void
 fetch_attachments (ECalBackendGroupwise *cbgw, ECalComponent *comp)
 {
 	GSList *attach_list = NULL, *new_attach_list = NULL;
 	GSList *l;
-	char  *attach_store, *filename, *file_contents;
+	char  *attach_store;
 	char *dest_url, *dest_file;
-	int fd, len;
-	int len_read = 0;
-	char buf[1024];
-	struct stat sb;
+	int fd;
 	const char *uid;
 
-
 	e_cal_component_get_attachment_list (comp, &attach_list);
 	e_cal_component_get_uid (comp, &uid);
 	/*FIXME  get the uri rather than computing the path */
-	attach_store = g_strconcat
-			(e_cal_backend_groupwise_get_local_attachments_store (cbgw), NULL);
+	attach_store = g_strdup (e_cal_backend_groupwise_get_local_attachments_store (cbgw));
 	
 	for (l = attach_list; l ; l = l->next) {
 		char *sfname = (char *)l->data;
+		char *filename, *new_filename;
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		GMappedFile *mapped_file;
+#else
+		char *file_contents;
+		int len;
+#endif
+		GError *error = NULL;
 
-		filename = g_strrstr (sfname, "/") + 1;	
-
-		// open the file using the data
-		fd = open (sfname, O_RDONLY); 
-		if (fd == -1) {
-			/* TODO handle error conditions */
-			g_message ("DEBUG: could not open the file descriptor\n");
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		mapped_file = g_mapped_file_new (sfname, FALSE, &error);
+		if (!mapped_file) {
+			g_message ("DEBUG: could not map %s: %s\n",
+				   sfname, error->message);
+			g_error_free (error);
 			continue;
 		}
-		if (fstat (fd, &sb) == -1) {
-			/* TODO handle error conditions */
-			g_message ("DEBUG: could not fstat the attachment file\n");
+#else
+		if (!g_file_get_contents (sfname, &file_contents, &len, &error)) {
+			g_message ("DEBUG: could not read %s: %s\n",
+				   sfname, error->message);
+			g_error_free (error);
 			continue;
 		}
-		len = sb.st_size;
-
-		file_contents = g_malloc (len + 1);
-	
-		while (len_read < len) {
-			int c = read (fd, buf, sizeof (buf));
-
-			if (c == -1)
-				break;
-
-			memcpy (&file_contents[len_read], buf, c);
-			len_read += c;
-		}
-		file_contents [len_read] = 0;
-
-		/* write*/
-		dest_file = g_strconcat (attach_store, "/", uid, "-",
-				filename, NULL);
-		fd = open (dest_file, O_RDWR|O_CREAT|O_TRUNC, 0600);
+#endif
+		filename = g_path_get_basename (sfname);
+		new_filename = g_strconcat (uid, "-", filename, NULL);
+		g_free (filename);
+		dest_file = g_build_filename (attach_store, new_filename, NULL);
+		g_free (new_filename);
+		fd = g_open (dest_file, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, 0600);
 		if (fd == -1) {
 			/* TODO handle error conditions */
-			g_message ("DEBUG: could not serialize attachments\n");
-		}
-
-		if (write (fd, file_contents, len_read) == -1) {
+			g_message ("DEBUG: could not open %s for writing\n",
+				   dest_file);
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		} else if (write (fd, g_mapped_file_get_contents (mapped_file),
+				  g_mapped_file_get_length (mapped_file)) == -1) {
+#else
+		} else if (write (fd, file_contents, len) == -1) {
+#endif
 			/* TODO handle error condition */
 			g_message ("DEBUG: attachment write failed.\n");
 		}
 
-		dest_url = g_strconcat ("file:///", dest_file, NULL);
-		new_attach_list = g_slist_append (new_attach_list, dest_url);
+#if GLIB_CHECK_VERSION (2, 8, 0)
+		g_mapped_file_free (mapped_file);
+#else
+		g_free (file_contents);
+#endif
+		close (fd);
+		dest_url = g_filename_to_uri (dest_file, NULL, NULL);
 		g_free (dest_file);
+		new_attach_list = g_slist_append (new_attach_list, dest_url);
 	}
 	g_free (attach_store);
 	e_cal_component_set_attachment_list (comp, new_attach_list);
 }
 
+static void 
+change_status (ECalComponent *comp, icalparameter_partstat status, const char *email)
+{	
+	icalproperty *prop;	
+	icalparameter *param;
+	gboolean found = FALSE;
+	icalcomponent *icalcomp = e_cal_component_get_icalcomponent (comp);
+
+
+	for (prop = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
+			prop;
+			prop = icalcomponent_get_next_property (icalcomp, ICAL_ATTENDEE_PROPERTY)) {
+		const char *attendee = icalproperty_get_attendee (prop);
+
+		if (!g_strncasecmp (attendee, "mailto:", 7))
+			attendee += 7;
+
+		if (!g_ascii_strcasecmp (attendee, email)) {
+			found = TRUE;
+			param = icalparameter_new_partstat (status);
+			icalproperty_set_parameter (prop, param);
+			break;
+		}
+	}
+
+	/* We couldn find the attendee in the component, so add a new attendee */
+	if (!found) {
+		char *temp = g_strdup_printf ("MAILTO:%s", email);
+
+		prop = icalproperty_new_attendee ((const char *) temp);
+		icalcomponent_add_property (icalcomp, prop);
+
+		param = icalparameter_new_partstat (ICAL_PARTSTAT_DELEGATED);
+		icalproperty_add_parameter (prop, param);
+
+		param = icalparameter_new_role (ICAL_ROLE_NONPARTICIPANT);
+		icalproperty_add_parameter (prop, param);
+
+		param = icalparameter_new_cutype (ICAL_CUTYPE_INDIVIDUAL);
+		icalproperty_add_parameter (prop, param);
+
+		param = icalparameter_new_rsvp (ICAL_RSVP_TRUE);
+		icalproperty_add_parameter (prop, param);
+
+		g_free (temp);
+	}
+}
+
 static ECalBackendSyncStatus
 receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalcomp)
 {
@@ -2117,10 +2197,31 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 	ECalBackendGroupwisePrivate *priv;
 	icalproperty_method method;
 	EGwConnectionStatus status;
-	gboolean remove = FALSE;
+	gboolean all_instances = FALSE;
+	icalparameter_partstat pstatus;
+	icalproperty *icalprop;
 
 	priv = cbgw->priv;
 
+	/* When the icalcomponent is obtained through the itip message rather
+	 * than from the SOAP protocol, the container id has to be explicitly 
+	 * added to the xgwrecordid inorder to obtain the item id. */
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
+	while (icalprop) {
+		const char *x_name;
+
+		x_name = icalproperty_get_x_name (icalprop);
+		if (!strcmp (x_name, "X-GW-RECUR-INSTANCES-MOD-TYPE")) {
+			if (!strcmp (icalproperty_get_x (icalprop), "All")) {
+				all_instances = TRUE;
+				icalcomponent_remove_property (icalcomp, icalprop);
+				break;
+			}
+		}
+
+		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
+	}
+
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp));
 	method = icalcomponent_get_method (icalcomp);
@@ -2129,56 +2230,80 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 	if (e_cal_component_has_attachments (comp))
 		fetch_attachments (cbgw, comp);
 
-	status = e_gw_connection_send_appointment (cbgw, priv->container_id, comp, method, &remove, &modif_comp);
+	status = e_gw_connection_send_appointment (cbgw, priv->container_id, comp, method, all_instances, &modif_comp, &pstatus);
 
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
-		status = e_gw_connection_send_appointment (cbgw, priv->container_id, comp, method, &remove, &modif_comp);
+		status = e_gw_connection_send_appointment (cbgw, priv->container_id, comp, method, all_instances, &modif_comp, &pstatus);
 
-	if (status == E_GW_CONNECTION_STATUS_ITEM_ALREADY_ACCEPTED) {
-		g_object_unref (comp);
-		return GNOME_Evolution_Calendar_Success;
-	}
+	if (!modif_comp)
+		modif_comp = g_object_ref (comp);
 
 	/* update the cache */
-	if (status == E_GW_CONNECTION_STATUS_OK) {
-		if (remove) {
+	if (status == E_GW_CONNECTION_STATUS_OK || status == E_GW_CONNECTION_STATUS_ITEM_ALREADY_ACCEPTED) {
+		GSList *comps = NULL, *l;
+		gboolean found = FALSE;
+
+		if (all_instances) {
 			const char *uid;
-			
-			e_cal_component_get_uid (comp, (const char **) &uid);
-			if (e_cal_backend_cache_remove_component (priv->cache, uid, NULL))
-				e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), uid, e_cal_component_get_as_string (comp), NULL);
+ 			
+			e_cal_component_get_uid (modif_comp, (const char **) &uid);
+			comps = e_cal_backend_cache_get_components_by_uid (priv->cache, uid);
+
+			if (!comps)
+				comps = g_slist_append (comps, g_object_ref (modif_comp));
+			else
+				found = TRUE;
+		} else {
+			ECalComponentId *id = e_cal_component_get_id (modif_comp);
+			ECalComponent *comp = NULL;
+
+			comp = e_cal_backend_cache_get_component (priv->cache, id->uid, id->rid);
+
+			if (!comp)
+				comps = g_slist_append (comps, g_object_ref (modif_comp));
+			else {
+				comps = g_slist_append (comps, comp);
+				found = TRUE;
+ 			}
+ 
+			e_cal_component_free_id (id);
 		}
-		else {
-			char *cache_comp = NULL, *uid, *new_comp = NULL;
-			ECalComponent *cache_component;
-			
-			if (!modif_comp)
-				modif_comp = g_object_ref (comp);
-			else
-				e_cal_component_commit_sequence (modif_comp);
+ 
+		for (l = comps; l != NULL; l = l->next) {
+			ECalComponent *comp = E_CAL_COMPONENT (l->data);
 
-			e_cal_component_get_uid (modif_comp, (const char **) &uid);	
-			cache_component = e_cal_backend_cache_get_component (priv->cache, uid, NULL);
-			
-			if (cache_component) {
-				e_cal_component_commit_sequence (cache_component);
-				cache_comp = e_cal_component_get_as_string (cache_component);
-			}
+			if (pstatus == ICAL_PARTSTAT_DECLINED) {
+				ECalComponentId *id = e_cal_component_get_id (comp);
 
-			e_cal_backend_cache_put_component (priv->cache, modif_comp);	
-			new_comp = e_cal_component_get_as_string (modif_comp);
+				if (e_cal_backend_cache_remove_component (priv->cache, id->uid, id->rid)) {
 
-			if (cache_comp)
-				e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp, new_comp);
-			else
-				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), new_comp);
+					e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), id, e_cal_component_get_as_string (comp), NULL);
+				e_cal_component_free_id (id);
+
+				}
+
+			} else {
+				char *comp_str = NULL;
+
+				change_status (comp, pstatus, e_gw_connection_get_user_email (priv->cnc));
+				e_cal_backend_cache_put_component (priv->cache, comp);	
+				comp_str = e_cal_component_get_as_string (comp);
 				
-			g_free (cache_comp);
-			g_free (new_comp);
-			g_object_unref (modif_comp);
-		}
+				if (found)
+					e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), comp_str, comp_str);
+				else
+					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), comp_str);
+
+				g_free (comp_str);
+			}
+ 		}
+		
+		g_slist_foreach (comps, (GFunc) g_object_unref, NULL);
+		g_slist_free (comps);
 		g_object_unref (comp);	
-		return GNOME_Evolution_Calendar_Success;
+		g_object_unref (modif_comp);
+ 		return GNOME_Evolution_Calendar_Success;
+
 	}
 
 	if (status == E_GW_CONNECTION_STATUS_INVALID_OBJECT) {
