@@ -446,6 +446,31 @@ e_cal_backend_sync_get_object_list (ECalBackendSync *backend, EDataCal *cal, con
  	return status;
 }
 
+/**
+ * e_cal_backend_sync_get_attachment_list:
+ * @backend: An ECalBackendSync object.
+ * @cal: An EDataCal object.
+ * @uid: Unique id of the calendar object.
+ * @rid: Recurrence id of the calendar object.
+ * @attachments: Placeholder for list of returned attachment uris.
+ *
+ * Calls the get_attachment_list method on the given backend.
+ *
+ * Return value: Status code.
+ */
+ECalBackendSyncStatus
+e_cal_backend_sync_get_attachment_list (ECalBackendSync *backend, EDataCal *cal, const char *uid, const char *rid, GSList **attachments)
+{
+ 	ECalBackendSyncStatus status;
+ 
+  	g_return_val_if_fail (backend && E_IS_CAL_BACKEND_SYNC (backend), GNOME_Evolution_Calendar_OtherError);
+  	g_return_val_if_fail (attachments, GNOME_Evolution_Calendar_OtherError);
+  
+ 	LOCK_WRAPPER (get_attachment_list_sync, (backend, cal, uid, rid, attachments));
+  
+ 	return status;
+}
+
 /**
  * e_cal_backend_sync_get_timezone:
  * @backend: An ECalBackendSync object.
@@ -698,10 +723,20 @@ _e_cal_backend_remove_object (ECalBackend *backend, EDataCal *cal, const char *u
 		    == GNOME_Evolution_Calendar_Success) {
 			e_data_cal_notify_object_modified (cal, status, old_object, calobj);
 			g_free (calobj);
-		} else
-			e_data_cal_notify_object_removed (cal, status, uid, old_object, object);
+		} else {
+			ECalComponentId *id = g_new0 (ECalComponentId, 1);
+			id->uid = g_strdup (uid);
+			
+			if (mod == CALOBJ_MOD_THIS)
+				id->rid = g_strdup (rid);
+
+			e_data_cal_notify_object_removed (cal, status, id, old_object, object);
+
+			e_cal_component_free_id (id);
+			
+		}
 	} else
-		e_data_cal_notify_object_removed (cal, status, uid, old_object, object);
+		e_data_cal_notify_object_removed (cal, status, NULL, old_object, object);
 
 	g_free (object);
 }
@@ -767,6 +802,20 @@ _e_cal_backend_get_object (ECalBackend *backend, EDataCal *cal, const char *uid,
 	g_free (object);
 }
 
+static void
+_e_cal_backend_get_attachment_list (ECalBackend *backend, EDataCal *cal, const char *uid, const char *rid)
+{
+	ECalBackendSyncStatus status;
+	GSList *list = NULL;
+
+	status = e_cal_backend_sync_get_attachment_list (E_CAL_BACKEND_SYNC (backend), cal, uid, rid, &list);
+
+	e_data_cal_notify_attachment_list (cal, status, list);
+	
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_free (list);
+}
+
 static void
 _e_cal_backend_get_object_list (ECalBackend *backend, EDataCal *cal, const char *sexp)
 {
@@ -908,6 +957,7 @@ e_cal_backend_sync_class_init (ECalBackendSyncClass *klass)
 	backend_class->get_default_object = _e_cal_backend_get_default_object;
 	backend_class->get_object = _e_cal_backend_get_object;
 	backend_class->get_object_list = _e_cal_backend_get_object_list;
+	backend_class->get_attachment_list = _e_cal_backend_get_attachment_list;
 	backend_class->get_timezone = _e_cal_backend_get_timezone;
 	backend_class->add_timezone = _e_cal_backend_add_timezone;
 	backend_class->set_default_timezone = _e_cal_backend_set_default_timezone;
