@@ -359,7 +359,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 	/* This makes the cleanup easier. We just unref it each time
 	 * we find it in account->priv->folders.
 	 */
-	//d(printf ("adding folder : %s\n", e_folder_get_name (folder)));
 	key = (char *) e_folder_exchange_get_path (folder);
 	if (!g_hash_table_lookup (account->priv->folders, key)) {
 		/* Avoid dupilcations since the user could add a folder as
@@ -368,7 +367,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		g_hash_table_insert (account->priv->folders,
 				     key,
 				     folder);
-		//d(printf ("added path : %s\n", key));
 		table_updated = 1;
 	}
 
@@ -387,7 +385,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		g_hash_table_insert (account->priv->folders,
 				     key,
 				     folder);
-		//d(printf ("added phy path : %s\n", key));
 		table_updated = 1;
 	}
 
@@ -400,7 +397,6 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		g_hash_table_insert (account->priv->folders,
 				     key,
 				     folder);
-		//d(printf ("added int path : %s\n", key));
 		table_updated = 1;
 	}
 
@@ -1359,7 +1355,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	E2kHTTPStatus status;
 	gboolean redirected = FALSE;
 	E2kResult *results;
-	int nresults, mode;
+	int nresults = 0, mode;
 	GByteArray *entryid;
 	const char *timezone;
 	E2kGlobalCatalogStatus gcstatus;
@@ -1543,6 +1539,8 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	if (!setup_account_hierarchies (account)) {
 		*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
 		g_mutex_unlock (account->priv->connect_lock);
+		if (nresults)
+			e2k_results_free (results, nresults);
 		return NULL; /* FIXME: what error has happened? */
 	}
 
@@ -1583,6 +1581,8 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 
 	g_signal_emit (account, signals[CONNECTED], 0, account->priv->ctx);
 	g_mutex_unlock (account->priv->connect_lock);
+	if (nresults)
+		e2k_results_free (results, nresults);
 	return account->priv->ctx;
 }
 
